/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*       "EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.      *
*                                                                       *
*  Copyright (C) 1989, 1992 Lugaru Software Ltd.  All rights reserved.  *
*                                                                       *
* Limited permission is hereby granted to reproduce and modify this     *
* copyrighted material provided that the resulting code is used only in *
* conjunction with Lugaru products and that this notice is retained in  *
* any such reproduction or modification.                                *
************************************************************************/

#include "eel.h"

/* a screen worth of window info */
struct screen_info_3 {
    struct window_info *wins[36];   /* window information structs */
    int lines[36], cols[36];        /* size of screen */
    int num[36];                    /* # of tiled window on screen */
    int cur_window[36];             /* # of current window */
    int echo_line[36];              // line for echo area
    int avoid_top[36];              // # of lines at top with no tiled windows
    int avoid_bottom[36];           // # of lines at bottom with no tiled windows
    int permanent_menu[36];         // want perm menu?
} zoom_info_3;

int num_zooms = 0;
int _z00m_buffer, save_scrn_like_zoom = 0;

save_screen_2(p)          /* save current window info in p */
struct screen_info_3 *p;  /* on first call with p, p->wins must be 0 */
{
    int i, oldroom = 0, nwins = number_of_windows() - number_of_popups();

    p->lines[num_zooms] = screen_lines, p->cols[num_zooms] = screen_cols;
    p->cur_window[num_zooms] = window_number;
    p->echo_line[num_zooms] = echo_line;
    p->avoid_top[num_zooms] = avoid_top_lines;
    p->avoid_bottom[num_zooms] = avoid_bottom_lines;
    p->permanent_menu[num_zooms] = permanent_menu;
    if (p->wins[num_zooms])
        oldroom = ptrlen(p->wins[num_zooms]) / sizeof(struct window_info);
    if (oldroom < nwins)
    {
        if (p->wins[num_zooms])
            free(p->wins[num_zooms]);
        p->wins[num_zooms] = (struct window_info *) malloc(nwins
                  * sizeof(struct window_info));
    }
    for (i = 0; i < nwins; i++)
        get_window_info(i, p->wins[num_zooms] + i);
    p->num[num_zooms] = i;
}

restore_screen_2(p)       /* restore windows from data in p */
struct screen_info_3 *p;
{
    int i, origbuf, origpt;

    bufnum = origbuf = window_bufnum;
    origpt = point;
    if (screen_lines != p->lines[num_zooms] || screen_cols != p->cols[num_zooms])
        error("Screen is in wrong mode for restore.");
    for (i = 0; i < p->num[num_zooms]; i++)
        window_create(!i, p->wins[num_zooms] + i);
    echo_line = p->echo_line[num_zooms];
    select_low_window(p->cur_window[num_zooms], p->avoid_top[num_zooms], p->avoid_bottom[num_zooms]);
    restore_menu(p->permanent_menu[num_zooms]);
    if (bufnum == origbuf)
        point = origpt;
}

command screen_save()
{
    int i;
    char msg[40], dig;

    if (!exist("-z00m"))
        setup_z00m();
    grab_full_line (_z00m_buffer, msg);
    do
    {
        sayput ("Save screen to [A-Z] [%s",msg);
        dig = getkey();
        check_abort();
    } while ((!isalpha(dig)) && (dig != 27));
    if (dig == 27)
    {
        say ("");
        return;
    }
    dig = toupper(dig);
    i = (dig - 'A') + 10;
    save_var num_zooms = i;
    save_screen_2(&zoom_info_3);
    if (save_scrn_like_zoom)
        one_window();
    display_column = display_column.default;
    save_var bufnum = _z00m_buffer;
    replace (i, dig);
    say ("");
    iter = 1;
}

command screen_restore()
{
    int i;
    char msg[40], dig, dig2;

    if (!exist("-z00m"))
        setup_z00m();
    grab_full_line (_z00m_buffer, msg);
    do
    {
        sayput ("Restore screen from [0-9, A-Z] [%s",msg);
        dig = getkey();
        check_abort();
    } while ((!isdigit(dig)) && (!isalpha(dig)) && (dig != 27));
    if (dig == 27)
    {
        say ("");
        return;
    }
    if (isdigit(dig))
    {
        i = dig - '0';
        save_var num_zooms = i;
        if (msg[i] != ' ')
            restore_screen_2(&zoom_info_3);
        else
            say ("No screen saved to %c",dig);
    }
    else
    {
        if ((num_zooms < 0) || (num_zooms > 9))
            num_zooms = 0;
        save_screen_2(&zoom_info_3);
        dig2 = num_zooms + '0';
        save_var bufnum = _z00m_buffer;
        replace (num_zooms, dig2);
        ++num_zooms;
        dig = toupper(dig);
        i = (dig - 'A') + 10;
        if (isupper(msg[i]))
        {
            save_var num_zooms = i;
            restore_screen_2(&zoom_info_3);
        }
        else
            say ("No screen saved to %c",dig);
    }
    say ("");
    iter = 1;
}

setup_z00m()
{
    save_var bufnum = _z00m_buffer = zap ("-z00m");
    stuff ("                                    ]: ");
    point = num_zooms = 0;
}

get_window_info(win, p)
struct window_info *p;
{
    low_window_info(win, p);
    save_var window_handle = win;
    p->dcolumn = display_column;
    p->prevbuf = _prev_buffer;
}

window_create(first, p)
struct window_info *p;
{
    low_window_create(first, p);
    display_column = p->dcolumn;
    _prev_buffer = p->prevbuf;
}
